"""本模块提供并行执行器, 及方便函数 `io_bound`, `cpu_bound`.
"""
import asyncio
import functools
import importlib
from asyncio.events import AbstractEventLoop
from concurrent.futures import ProcessPoolExecutor, ThreadPoolExecutor
from typing import Awaitable, Callable, ClassVar, Dict, Tuple, TypeVar

from typing_extensions import ParamSpec

P = ParamSpec("P")

R = TypeVar("R")


class ParallelExecutor:
    """并行执行器.

    推荐使用 Ariadne.create 以自动绑定事件循环.
    """

    thread_exec: ThreadPoolExecutor
    proc_exec: ProcessPoolExecutor
    loop_ref_dict: ClassVar[Dict[AbstractEventLoop, "ParallelExecutor"]] = {}
    func_mapping: ClassVar[Dict[str, Callable[P, R]]] = {}

    def __init__(
        self,
        loop: AbstractEventLoop = None,
        max_thread: int = None,
        max_process: int = None,
    ):
        """初始化并行执行器.

        Args:
            loop (AbstractEventLoop, optional): 要绑定的事件循环. Defaults to None.
            max_thread (int, optional): 最大线程数. Defaults to None.
            max_process (int, optional): 最大进程数. Defaults to None.

        `max_thread` 与 `max_process` 参数默认值请参阅 `concurrent.futures`.
        """
        self.thread_exec = ThreadPoolExecutor(max_workers=max_thread)
        self.proc_exec = ProcessPoolExecutor(max_workers=max_process)
        if loop:
            self.bind_loop(loop)

    @classmethod
    def get(cls, loop: AbstractEventLoop):
        if loop not in cls.loop_ref_dict:
            cls.loop_ref_dict[loop] = ParallelExecutor()
        return cls.loop_ref_dict[loop]

    def bind_loop(self, loop: AbstractEventLoop):
        self.loop_ref_dict[loop] = self

    @classmethod
    def shutdown(cls):
        for exec in cls.loop_ref_dict.values():
            exec.thread_exec.shutdown()
            exec.proc_exec.shutdown()

    @classmethod
    def run_func(cls, name: str, module: str, args: tuple, kwargs: dict) -> R:
        importlib.import_module(module)
        return cls.func_mapping[name](*args, **kwargs)


def io_bound(func: Callable[P, R]) -> Callable[P, Awaitable[R]]:
    ParallelExecutor.func_mapping[func.__qualname__] = func

    @functools.wraps(func)
    async def wrapper(*args: P.args, **kwargs: P.kwargs) -> R:
        loop = asyncio.get_running_loop()
        executor = ParallelExecutor.get(loop)
        return await loop.run_in_executor(
            executor.thread_exec,
            ParallelExecutor.run_func,
            func.__qualname__,
            func.__module__,
            args,
            kwargs,
        )

    return wrapper


def cpu_bound(func: Callable[P, R]) -> Callable[P, Awaitable[R]]:
    ParallelExecutor.func_mapping[func.__qualname__] = func

    @functools.wraps(func)
    async def wrapper(*args: P.args, **kwargs: P.kwargs) -> R:
        loop = asyncio.get_running_loop()
        executor = ParallelExecutor.get(loop)
        return await loop.run_in_executor(
            executor.proc_exec,
            ParallelExecutor.run_func,
            func.__qualname__,
            func.__module__,
            args,
            kwargs,
        )

    return wrapper



import argparse
import inspect
import re
from typing import TYPE_CHECKING, List, NoReturn, Type, Union

from ..chain import MessageChain

if TYPE_CHECKING:
    from .pattern import ArgumentMatch


def split(string: str) -> List[str]:
    result: List[str] = []
    quote = ""
    cache: List[str] = []
    for index, char in enumerate(string):
        if char in "'\"":
            if not quote:
                quote = char
            elif char == quote and index and string[index - 1] != "\\":  # is current quote, not transfigured
                quote = ""
            else:
                cache.append(char)
            continue
        if not quote and char == " ":
            result.append("".join(cache))
            cache = []
        else:
            if char != "\\":
                cache.append(char)
    if cache:
        result.append("".join(cache))
    return result


def gen_flags_repr(flags: re.RegexFlag) -> str:
    flags_list: List[str] = []
    if re.ASCII in flags:
        flags_list.append("a")
    if re.IGNORECASE in flags_list:
        flags_list.append("i")
    if re.LOCALE in flags_list:
        flags_list.append("L")
    if re.MULTILINE in flags_list:
        flags_list.append("m")
    if re.DOTALL in flags_list:
        flags_list.append("s")
    if re.UNICODE in flags_list:
        flags_list.append("u")
    if re.VERBOSE in flags_list:
        flags_list.append("x")


class ArgumentMatchType:
    def __init__(self, match: "ArgumentMatch", regex: re.Pattern):
        self.match = match
        self.regex: re.Pattern = regex

    def __call__(self, string: str) -> MessageChain:
        if self.regex and not self.regex.fullmatch(string):
            raise ValueError(f"{string} not matching {self.regex.pattern}")
        return MessageChain.fromMappingString(string, self.match.elem_mapping_ctx.get())


class TwilightParser(argparse.ArgumentParser):
    def error(self, message) -> NoReturn:
        raise ValueError(message)

    def accept_type(self, action: Union[str, type]) -> bool:
        if isinstance(action, str):
            action_cls: Type[argparse.Action] = self._registry_get("action", action, action)
        elif issubclass(action, argparse.Action):
            action_cls = action
        else:
            return False
        action_init_sig = inspect.signature(action_cls.__init__)
        if "type" not in action_init_sig.parameters:
            return False
        return True



from contextlib import contextmanager
from contextvars import ContextVar
from typing import TYPE_CHECKING

from .model import UploadMethod

if TYPE_CHECKING:
    from asyncio.events import AbstractEventLoop

    from graia.broadcast import Broadcast

    from .adapter import Adapter
    from .app import Ariadne
    from .event import MiraiEvent


ariadne_ctx: ContextVar["Ariadne"] = ContextVar("application")
adapter_ctx: ContextVar["Adapter"] = ContextVar("adapter")
event_ctx: ContextVar["MiraiEvent"] = ContextVar("event")
event_loop_ctx: ContextVar["AbstractEventLoop"] = ContextVar("event_loop")
broadcast_ctx: ContextVar["Broadcast"] = ContextVar("broadcast")
upload_method_ctx: ContextVar["UploadMethod"] = ContextVar("upload_method")


@contextmanager
def enter_message_send_context(method: UploadMethod):
    t = upload_method_ctx.set(method)
    yield
    upload_method_ctx.reset(t)


@contextmanager
def enter_context(app: "Ariadne" = None, event: "MiraiEvent" = None):
    token_app = None
    token_event = None
    token_loop = None
    token_bcc = None
    token_adapter = None

    if app:
        token_app = ariadne_ctx.set(app)
        token_loop = event_loop_ctx.set(app.broadcast.loop)
        token_bcc = broadcast_ctx.set(app.broadcast)
        token_adapter = adapter_ctx.set(app.adapter)
    if event:
        token_event = event_ctx.set(event)

    yield

    try:
        if token_app:
            ariadne_ctx.reset(token_app)
        if token_adapter:
            adapter_ctx.reset(token_adapter)
        if token_event:
            event_ctx.reset(token_event)
        if token_loop:
            event_loop_ctx.reset(token_loop)
        if token_bcc:
            broadcast_ctx.reset(token_bcc)
    except ValueError:
        pass



import abc
import asyncio
import functools
import json
from asyncio.events import AbstractEventLoop
from asyncio.exceptions import CancelledError
from asyncio.locks import Event
from asyncio.tasks import Task
from typing import (
    TYPE_CHECKING,
    Any,
    Awaitable,
    Callable,
    Dict,
    NoReturn,
    Optional,
    Set,
    TypeVar,
    Union,
)

import aiohttp.web_exceptions
from aiohttp import ClientSession, FormData
from aiohttp.client_ws import ClientWebSocketResponse
from aiohttp.http_websocket import WSMsgType
from graia.broadcast import Broadcast
from graia.broadcast.utilles import run_always_await
from loguru import logger
from typing_extensions import Concatenate
from yarl import URL

from .context import enter_context
from .event import MiraiEvent
from .event.network import RemoteException
from .exception import InvalidArgument, InvalidSession, NotSupportedAction
from .model import CallMethod, DatetimeEncoder, MiraiSession
from .util import validate_response

if TYPE_CHECKING:
    from .app import Ariadne
    from .typing import P, R, Self


def require_verified(
    func: "Callable[Concatenate[Adapter, P], R]",
) -> "Callable[Concatenate[Adapter, P], R]":
    @functools.wraps(func)
    def wrapper(self: "Adapter", *args: "P.args", **kwargs: "P.kwargs") -> "R":
        if not self.mirai_session.session_key:
            raise InvalidSession("you must verify the session before action.")
        return func(self, *args, **kwargs)

    return wrapper


def error_wrapper(
    network_action_callable: "Callable[Concatenate[Self, P], Awaitable[R]]",
) -> "Callable[Concatenate[Self, P], Awaitable[R]]":
    @functools.wraps(network_action_callable)
    async def wrapped_network_action_callable(self: "Adapter", *args: "P.args", **kwargs: "P.kwargs") -> "R":
        running_count = 0

        while running_count < 5:
            running_count += 1
            try:
                result = await network_action_callable(self, *args, **kwargs)
            except InvalidSession as invalid_session_exc:
                logger.error("Invalid session detected, asking daemon to restart adapter...")
                logger.exception(invalid_session_exc)
                await self.stop()
            except aiohttp.web_exceptions.HTTPNotFound:
                raise NotSupportedAction(f"{network_action_callable.__name__}: this action not supported")
            except aiohttp.web_exceptions.HTTPInternalServerError as e:
                self.broadcast.postEvent(RemoteException())
                logger.error("An exception has thrown by remote, please check the console!")
                raise
            except (
                aiohttp.web_exceptions.HTTPMethodNotAllowed,
                aiohttp.web_exceptions.HTTPRequestURITooLong,
                aiohttp.web_exceptions.HTTPTooManyRequests,
            ):

                logger.error(
                    f"It seems that we post in a wrong way "
                    f"for the action '{network_action_callable.__name__}', please open a issue."
                )
                raise
            except aiohttp.web_exceptions.HTTPRequestTimeout:
                logger.error(
                    f"timeout on {network_action_callable.__name__}, retry after 5 seconds...".format()
                )
                await asyncio.sleep(5)
                raise
            else:
                return result
        raise TimeoutError(f"Failed after 5 try on {network_action_callable.__name__}.")

    return wrapped_network_action_callable


class Adapter(abc.ABC):
    """
    适配器抽象基类.

    Args:
        broadcast(Broadcast): Broadcast 实例
        session: Session 实例, 存储了连接信息
    """

    def __init__(self, broadcast: Broadcast, mirai_session: MiraiSession) -> None:
        self.broadcast = broadcast
        self.loop: AbstractEventLoop = broadcast.loop
        self.mirai_session: MiraiSession = mirai_session
        self.session: Optional[ClientSession] = None
        self.running: bool = False
        self.fetch_task: Optional[Task] = None
        self.app: Optional["Ariadne"] = None

    @abc.abstractmethod
    async def fetch_cycle(self) -> None:
        """
        负责接收并处理数据, 广播事件.
        """

    @abc.abstractmethod
    @require_verified
    @error_wrapper
    async def call_api(
        self, action: str, method: CallMethod, data: Optional[Union[dict, str]] = None
    ) -> Union[dict, list]:
        """
        向Mirai端发送数据.
        如有回复则应一并返回.

        Args:
            action (str): 要执行的操作.
            method (CallMethod): 指示对 mirai-api-http 端发送数据的方式.
            data (dict): 要发送的数据.
        Returns:
            dict: 响应字典.
        """

    async def build_event(self, data: dict) -> MiraiEvent:
        """
        从尚未明确指定事件类型的对象中获取事件的定义, 并进行解析

        Args:
            data (dict): 用 dict 表示的序列化态事件, 应包含有字段 `type` 以供分析事件定义.

        Raises:
            InvalidArgument: 目标对象中不包含字段 `type`
            ValueError: 没有找到对应的字段, 通常的, 这意味着应用获取到了一个尚未被定义的事件, 请报告问题.

        Returns:
            MiraiEvent: 已经被序列化的事件
        """
        event_type: Optional[str] = data.get("type")
        if not event_type or not isinstance(event_type, str):
            raise InvalidArgument("Unable to find 'type' field for automatic parsing")
        event_class: Optional[MiraiEvent] = self.broadcast.findEvent(event_type)  # type: ignore
        if not event_class:
            logger.error("An event is not recognized! Please report with your log to help us diagnose.")
            raise ValueError(f"Unable to find event: {event_type}", data)
        data = {k: v for k, v in data.items() if k != "type"}
        event = event_class.parse_obj(data)
        return event

    async def start(self):
        if not self.session:
            self.session = ClientSession()
        if not self.fetch_task or self.fetch_task.done():
            self.running = True
            self.fetch_task = self.loop.create_task(self.fetch_cycle(), name="ariadne_adapter_fetch_cycle")

    @property
    def session_activated(self) -> bool:
        return bool(self.mirai_session.session_key)

    async def stop(self):
        """
        停止适配器, 并等待 `fetch_cycle` 方法完成.
        """
        self.running = False
        if self.fetch_task and not self.fetch_task.done():
            try:
                await self.fetch_task
            except CancelledError:
                pass
        self.fetch_task = None
        self.mirai_session.session_key = None


class HttpAdapter(Adapter):
    """
    仅使用正向 HTTP 的适配器, 采用短轮询接收事件/消息.
    不推荐.
    Note: Working In Progress
    """

    def __init__(
        self,
        broadcast: Broadcast,
        mirai_session: MiraiSession,
        fetch_interval: float = 0.5,
    ) -> None:
        super().__init__(broadcast, mirai_session)
        self.fetch_interval = fetch_interval
        raise NotImplementedError("HTTP Adapter is not supported yet!")

    async def fetch_cycle(self):
        while self.running:
            await asyncio.sleep(self.fetch_interval)

    @require_verified
    @error_wrapper
    async def call_api(
        self,
        action: str,
        method: CallMethod,
        data: Optional[Union[Dict[str, Any], str]] = None,
    ) -> Union[dict, list]:
        if not data:
            data = dict()
        if not self.session:
            raise RuntimeError("Unable to get session!")

        if method == CallMethod.GET or method == CallMethod.RESTGET:
            if isinstance(data, str):
                data = json.loads(data)
            async with self.session.get(URL(self.mirai_session.url_gen(action)).with_query(data)) as response:
                response.raise_for_status()
                resp_json: dict = await response.json()

        elif method == CallMethod.POST or method == CallMethod.RESTPOST:
            if not isinstance(data, str):
                data = json.dumps(data, cls=DatetimeEncoder)
            async with self.session.post(self.mirai_session.url_gen(action), data=data) as response:
                response.raise_for_status()
                resp_json: dict = await response.json()

        else:  # MULTIPART
            form = FormData()
            if not isinstance(data, dict):
                raise ValueError("Data must be a dict in multipart call!")
            for k, v in data.items():
                form.add_fields(k, v)
            async with self.session.post(self.mirai_session.url_gen(action), data=form) as response:
                response.raise_for_status()
                resp_json: dict = await response.json()
        if "data" in resp_json:
            resp = resp_json["data"]
        else:
            resp = resp_json

        validate_response(resp)
        return resp


class WebsocketAdapter(Adapter):
    """
    仅使用正向 Websocket 的适配器.
    因 Mirai API HTTP 的实现, 部分功能不可用.
    """

    class SyncIdManager:
        allocated: Set[int] = {0}

        @classmethod
        def allocate(cls) -> int:
            new_id = max(cls.allocated) + 1
            cls.allocated.add(new_id)
            return new_id

        @classmethod
        def done(cls, sync_id: int) -> None:
            if sync_id in cls.allocated:
                cls.allocated.remove(sync_id)

    class CallResponse(Event):
        def __init__(self, *, loop: AbstractEventLoop = None) -> None:
            super().__init__(loop=loop)
            self.response: Optional[dict] = None

    def __init__(self, broadcast: Broadcast, mirai_session: MiraiSession, ping: bool = True) -> None:
        super().__init__(broadcast, mirai_session)
        self.ping = ping
        self.ping_task: Optional[Task] = None
        self.ws_conn: Optional[ClientWebSocketResponse] = None
        self.query_dict = {"verifyKey": mirai_session.verify_key}
        self.sync_event: Dict[int, WebsocketAdapter.CallResponse] = {}
        if not mirai_session.single_mode:
            self.query_dict["qq"] = mirai_session.account

    async def ws_ping(self, interval: float = 30.0) -> None:
        while self.running:
            try:
                try:
                    await self.ws_conn.ping()
                    logger.debug("websocket: ping")
                except Exception as e:
                    logger.exception(f"websocket: ping failed: {e!r}")
                else:
                    logger.debug(f"websocket: ping success, delay {interval}s")
                    await asyncio.sleep(interval)
            except asyncio.CancelledError:
                logger.debug("websocket: pinger exit")
                return

    @require_verified
    @error_wrapper
    async def call_api(
        self, action: str, method: CallMethod, data: Optional[Union[dict, str]] = None
    ) -> Union[dict, list]:
        data = data or dict()
        if not self.ws_conn:
            raise ValueError("no existing websocket connection")
        sync_id = self.SyncIdManager.allocate()
        event = self.CallResponse()
        self.sync_event[sync_id] = event
        content = {
            "syncId": sync_id,
            "command": action,
            "content": data,
        }
        if method == CallMethod.RESTGET:
            content["subCommand"] = "get"
        elif method == CallMethod.RESTPOST:
            content["subCommand"] = "update"
        elif method == CallMethod.MULTIPART:
            raise NotImplementedError(f"Unsupported operation for WebsocketAdapter: {method}")

        await self.ws_conn.send_str(json.dumps(content, cls=DatetimeEncoder))
        logger.debug(f"websocket：sent with sync id: {sync_id}")
        await event.wait()
        self.SyncIdManager.done(sync_id)
        value: dict = event.response
        del self.sync_event[sync_id]
        del event
        validate_response(value)
        if "data" in value:
            return value["data"]
        return value

    async def raw_data_parser(self, raw_data: dict) -> None:
        sync_id: str = raw_data["syncId"]
        received_data: dict = raw_data["data"]
        validate_response(received_data)
        if not self.mirai_session.session_key:
            if session_key := received_data.get("session", None):
                self.mirai_session.session_key = session_key
            return
        sync_id = int(sync_id)
        if sync_id not in self.SyncIdManager.allocated:
            event = await self.build_event(received_data)
            with enter_context(app=self.app, event=event):
                self.broadcast.postEvent(event)
        else:
            if sync_id in self.sync_event:
                response = self.sync_event[sync_id]
                response.response = received_data
                response.set()

    async def fetch_cycle(self) -> None:
        async with self.session.ws_connect(
            str(URL(self.mirai_session.url_gen("all")).with_query(self.query_dict)),
            autoping=False,
        ) as connection:
            logger.info("websocket: connected")
            self.ws_conn = connection

            if self.ping:
                self.ping_task = self.loop.create_task(self.ws_ping(), name="ariadne_adapter_ws_ping")
                logger.info("websocket: ping task created")
            try:
                while self.running:
                    ws_message = await connection.receive()
                    if ws_message.type is WSMsgType.TEXT:
                        original_data: dict = json.loads(ws_message.data)
                        await self.raw_data_parser(original_data)

                    elif ws_message.type is WSMsgType.CLOSED:
                        logger.info("websocket: connection has been closed.")
                        return
                    elif ws_message.type is WSMsgType.PONG:
                        logger.debug("websocket: received pong")
                    else:
                        logger.debug("websocket: unknown message type - {}".format(ws_message.type))
            except Exception as e:
                logger.exception(e)
            finally:
                if self.ping_task:
                    self.ping_task.cancel()
                    self.ping_task = None
                    logger.debug("websocket: ping task complete")
        logger.info("websocket: disconnected")


class CombinedAdapter(Adapter):
    """
    使用正向Websocket接收事件与消息, 用HTTP发送消息/操作的适配器.
    稳定与性能的平衡, 但需要 Mirai API HTTP 同时启用 `http` 与 `ws` 适配器.

    Args:
        bcc(Broadcast): Broadcast 实例
        session: Session 实例, 存储了连接信息
        ping(bool): 是否启用 ping 功能.
    """

    def __init__(self, broadcast: Broadcast, mirai_session: MiraiSession, ping: bool = True) -> None:
        super().__init__(broadcast, mirai_session)
        self.ping = ping
        self.ping_task: Optional[Task] = None
        self.ws_conn: Optional[ClientWebSocketResponse] = None
        self.query_dict = {"verifyKey": mirai_session.verify_key}
        if not mirai_session.single_mode:
            self.query_dict["qq"] = mirai_session.account

    ws_ping = WebsocketAdapter.ws_ping

    call_api = HttpAdapter.call_api

    async def raw_data_parser(self, raw_data: dict) -> None:
        received_data = raw_data["data"]
        validate_response(received_data)
        if not self.mirai_session.session_key:
            if session_key := received_data.get("session", None):
                self.mirai_session.session_key = session_key
            return
        event = await self.build_event(received_data)
        with enter_context(app=self.app, event=event):
            self.broadcast.postEvent(event)

    fetch_cycle = WebsocketAdapter.fetch_cycle


DefaultAdapter = CombinedAdapter


class DebugAdapter(DefaultAdapter):
    """
    Debugging adapter
    """

    async def build_event(self, data: dict) -> MiraiEvent:
        """
        从尚未明确指定事件类型的对象中获取事件的定义, 并进行解析

        Args:
            data (dict): 用 dict 表示的序列化态事件, 应包含有字段 `type` 以供分析事件定义.

        Raises:
            InvalidArgument: 目标对象中不包含字段 `type`
            ValueError: 没有找到对应的字段, 通常的, 这意味着应用获取到了一个尚未被定义的事件, 请报告问题.

        Returns:
            MiraiEvent: 已经被序列化的事件
        """
        try:
            event = await super().build_event(data)
        except ValueError as e:
            logger.error(f"{e.args[0]}\n{json.dumps(data, indent=4)}")
            raise
        else:
            logger.debug(event)
            return event



from typing import TYPE_CHECKING, Callable, Iterable, List, Optional, Tuple, Type, Union

from graia.broadcast.entities.decorator import Decorator
from graia.broadcast.interfaces.decorator import DecoratorInterface

from .chain import MessageChain
from .element import Element

if TYPE_CHECKING:
    from ..typing import Slice

    ElementClass = Union[Iterable[Type[Element]], Type[Element]]
    Item = Union[Slice[ElementClass, Optional[int], None], ElementClass]


class Component(Decorator):
    filter: Callable[[Element], bool]
    match_time: int = -1

    def __init__(
        self,
        filter: Union[Tuple[Type[Element], ...], Callable[[Element], bool]],
        match_time: int = -1,
    ) -> None:
        if not callable(filter):

            def matcher(element: Element) -> bool:
                return isinstance(element, filter)

            self.filter = matcher
        else:
            self.filter = filter
        self.match_time = match_time

    def __class_getitem__(cls, item: "Item") -> "Component":
        match_time = -1

        if isinstance(item, slice):
            element_filter = item.start
            match_time = item.stop or match_time
        else:
            element_filter = item

        if isinstance(element_filter, type):
            element_cls = (element_filter,)
        else:
            element_cls = tuple(element_filter)

        return cls(element_cls, match_time)

    def select(self, chain: MessageChain) -> MessageChain:
        result: List[Element] = []
        matched: int = 0
        for element in chain.__root__:
            if self.filter(element):
                result.append(element)
                matched += 1
            if matched == self.match_time:
                break
        return MessageChain(result, inline=True)

    def target(self, interface: DecoratorInterface) -> MessageChain:
        if not isinstance(interface.return_value, MessageChain):
            raise TypeError(f"Can't cast Component on {type(interface.return_value)}!")
        chain: MessageChain = interface.return_value
        return self.select(chain)



from graia.broadcast.entities.decorator import Decorator
from graia.broadcast.exceptions import ExecutionStop
from graia.broadcast.interfaces.decorator import DecoratorInterface

from ...event.message import MessageEvent
from ..chain import MessageChain
from ..element import Quote, Source


class DetectPrefix(Decorator):
    """前缀检测器"""

    pre = True

    def __init__(self, prefix: str) -> None:
        """初始化前缀检测器.

        Args:
            prefix (str): 要匹配的前缀
        """
        self.prefix = prefix

    def target(self, interface: DecoratorInterface):
        if not isinstance(interface.event, MessageEvent):
            raise ExecutionStop
        header = interface.event.messageChain.include(Quote, Source)
        rest: MessageChain = interface.event.messageChain.exclude(Quote, Source)
        if not rest.startswith(self.prefix):
            raise ExecutionStop
        result = rest.removeprefix(self.prefix)
        if interface.annotation is MessageChain:
            return header + result


class DetectSuffix(Decorator):
    """后缀检测器"""

    pre = True

    def __init__(self, suffix: str) -> None:
        """初始化后缀检测器.

        Args:
            suffix (str): 要匹配的后缀
        """
        self.suffix = suffix

    def target(self, interface: DecoratorInterface):
        if not isinstance(interface.event, MessageEvent):
            raise ExecutionStop
        header = interface.event.messageChain.include(Quote, Source)
        rest: MessageChain = interface.event.messageChain.exclude(Quote, Source)
        if not rest.endswith(self.suffix):
            raise ExecutionStop
        result = rest.removesuffix(self.suffix)
        if interface.annotation is MessageChain:
            return header + result



import abc
from base64 import b64decode, b64encode
from datetime import datetime
from enum import Enum
from json import dumps as j_dump
from pathlib import Path
from typing import TYPE_CHECKING, List, NoReturn, Optional, Union, overload

from pydantic import validator
from pydantic.fields import Field
from typing_extensions import ParamSpec

from ..context import adapter_ctx, upload_method_ctx
from ..exception import InvalidArgument
from ..model import AriadneBaseModel, UploadMethod, datetime_encoder
from ..util import wrap_bracket

if TYPE_CHECKING:
    from pydantic.typing import ReprArgs

    from .chain import MessageChain


class NotSendableElement(Exception):
    """
    指示一个元素是不可发送的.
    """


class Element(AriadneBaseModel, abc.ABC):
    """
    指示一个消息中的元素.
    type (str): 元素类型
    """

    type: str

    def __hash__(self):
        return hash((type(self),) + tuple(self.__dict__.values()))

    def asDisplay(self) -> str:
        return ""

    def asPersistentString(self) -> str:
        return f"[mirai:{self.type}:{wrap_bracket(j_dump(self.dict(exclude={'type'})))}]"

    def prepare(self) -> None:
        """
        为元素被发送进行准备,
        若无异常被引发, 则完成本方法后元素应可被发送.

        保留空实现以允许不需要 `prepare`的元素类型存在.

        若本元素设计时便不可被发送, 请引发 `NotSendableElement` 异常.
        """

    def __repr_args__(self) -> "ReprArgs":
        return [(k, v) for k, v in self.dict(exclude={"type"}).items()]

    def __str__(self) -> str:
        return self.asDisplay()


class Plain(Element):
    type: str = "Plain"
    text: str

    def __init__(self, text: str, **kwargs) -> None:
        """实例化一个 Plain 消息元素, 用于承载消息中的文字.

        Args:
            text (str): 元素所包含的文字
        """
        super().__init__(text=text, **kwargs)

    def asDisplay(self) -> str:
        return self.text

    def asPersistentString(self) -> str:
        return self.text


class Source(Element):
    "表示消息在一个特定聊天区域内的唯一标识"
    type: str = "Source"
    id: int
    time: datetime

    class Config:
        json_encoders = {
            datetime: datetime_encoder,
        }

    def prepare(self) -> NoReturn:
        raise NotSendableElement

    def asPersistentString(self) -> str:
        return ""

    async def fetchOriginal(self) -> "MessageChain":
        """尝试从本元素恢复原本的消息链, 有可能失败.

        Returns:
            MessageChain: 原来的消息链.
        """
        from ..context import ariadne_ctx

        ariadne = ariadne_ctx.get()

        return (await ariadne.getMessageFromId(self.id)).messageChain


class Quote(Element):
    "表示消息中回复其他消息/用户的部分, 通常包含一个完整的消息链(`origin` 属性)"
    type: str = "Quote"
    id: int
    groupId: int
    senderId: int
    targetId: int
    origin: "MessageChain"

    @validator("origin", pre=True, allow_reuse=True)
    def _(cls, v):
        from .chain import MessageChain

        return MessageChain(v)  # no need to parse objects, they are universal!

    def prepare(self) -> NoReturn:
        raise NotSendableElement

    def asPersistentString(self) -> str:
        return ""


class At(Element):
    """该消息元素用于承载消息中用于提醒/呼唤特定用户的部分."""

    type: str = "At"
    target: int
    display: Optional[str] = None

    def __init__(self, target: int, **kwargs) -> None:
        """实例化一个 At 消息元素, 用于承载消息中用于提醒/呼唤特定用户的部分.

        Args:
            target (int): 需要提醒/呼唤的特定用户的 QQ 号(或者说 id.)
        """
        super().__init__(target=target, **kwargs)

    def __eq__(self, other: "At"):
        return isinstance(other, At) and self.target == other.target

    def prepare(self) -> None:
        try:
            if upload_method_ctx.get() != UploadMethod.Group:
                raise InvalidArgument(
                    "you cannot use this element in this method: {0}".format(upload_method_ctx.get().value)
                )
        except LookupError:
            pass

    def asDisplay(self) -> str:
        return f"@{self.display}" if self.display else f"@{self.target}"


class AtAll(Element):
    "该消息元素用于群组中的管理员提醒群组中的所有成员"
    type: str = "AtAll"

    def asDisplay(self) -> str:
        return "@全体成员"

    def prepare(self) -> None:
        try:
            if upload_method_ctx.get() != UploadMethod.Group:
                raise InvalidArgument(
                    "you cannot use this element in this method: {0}".format(upload_method_ctx.get().value)
                )
        except LookupError:
            pass


class Face(Element):
    "表示消息中所附带的表情, 这些表情大多都是聊天工具内置的."
    type: str = "Face"
    faceId: Optional[int] = None
    name: Optional[str] = None

    def asDisplay(self) -> str:
        return f"[表情:{f'{self.name}' if self.name else {self.faceId}}]"

    def __eq__(self, other: "Face") -> bool:
        return isinstance(other, Face) and (self.faceId == other.faceId or self.name == other.name)


class Xml(Element):
    "表示消息中的 XML 消息元素"
    type = "Xml"
    xml: str

    def asDisplay(self) -> str:
        return "[XML消息]"


class Json(Element):
    "表示消息中的 JSON 消息元素"
    type = "Json"
    Json: str = Field(..., alias="json")

    def __init__(self, json: Union[dict, str], **kwargs) -> None:
        if isinstance(json, dict):
            json = j_dump(json)
        super().__init__(json=json, **kwargs)

    def dict(self, *args, **kwargs):
        return super().dict(*args, **({**kwargs, "by_alias": True}))

    def asDisplay(self) -> str:
        return "[JSON消息]"


class App(Element):
    "表示消息中自带的 App 消息元素"
    type = "App"
    content: str

    def asDisplay(self) -> str:
        return "[APP消息]"


class PokeMethods(Enum):
    "戳一戳可用方法"
    ChuoYiChuo = "ChuoYiChuo"
    BiXin = "BiXin"
    DianZan = "DianZan"
    XinSui = "XinSui"
    LiuLiuLiu = "LiuLiuLiu"
    FangDaZhao = "FangDaZhao"
    BaoBeiQiu = "BaoBeiQiu"
    Rose = "Rose"
    ZhaoHuanShu = "ZhaoHuanShu"
    RangNiPi = "RangNiPi"
    JeiYin = "JeiYin"
    ShouLei = "ShouLei"
    GouYin = "GouYin"
    ZhuaYiXia = "ZhuaYiXia"
    SuiPing = "SuiPing"
    QiaoMen = "QiaoMen"


class Poke(Element):
    "表示消息中戳一戳消息元素"
    type = "Poke"
    name: PokeMethods

    def asDisplay(self) -> str:
        return f"[戳一戳:{self.name}]"


class Dice(Element):
    "表示消息中骰子消息元素"
    type = "Dice"
    value: int

    def asDisplay(self) -> str:
        return f"[骰子:{self.value}]"


class MusicShare(Element):
    "表示消息中音乐分享消息元素"
    type = "MusicShare"
    kind: Optional[str]
    title: Optional[str]
    summary: Optional[str]
    jumpUrl: Optional[str]
    pictureUrl: Optional[str]
    musicUrl: Optional[str]
    brief: Optional[str]

    def asDisplay(self) -> str:
        return f"[音乐分享:{self.title}]"


class ForwardNode(AriadneBaseModel):
    "表示合并转发中的一个节点"
    senderId: int
    time: datetime
    senderName: str
    messageChain: Optional["MessageChain"]
    messageId: Optional[int]

    class Config:
        json_encoders = {
            datetime: datetime_encoder,
        }


class Forward(Element):
    """
    指示合并转发信息

    nodeList (List[ForwardNode]): 转发的消息节点
    """

    type = "Forward"
    nodeList: List[ForwardNode]

    def asDisplay(self) -> str:
        return f"[合并转发:共{len(self.nodeList)}条]"

    def asPersistentString(self) -> str:
        return ""


class File(Element):
    "指示一个文件信息元素"
    type = "File"
    id: str
    name: str
    size: int

    def asDisplay(self) -> str:
        return f"[文件:{self.name}]"

    def asPersistentString(self) -> str:
        return ""


class ImageType(Enum):
    Friend = "Friend"
    Group = "Group"
    Temp = "Temp"
    Unknown = "Unknown"


image_upload_method_type_map = {
    UploadMethod.Friend: ImageType.Friend,
    UploadMethod.Group: ImageType.Group,
    UploadMethod.Temp: ImageType.Temp,
}

P = ParamSpec("P")


class MultimediaElement(Element):
    """指示多媒体消息元素."""

    id: Optional[str]
    url: Optional[str] = None
    base64: Optional[str] = None

    def __init__(
        self,
        id: Optional[str] = None,
        url: Optional[str] = None,
        *,
        path: Optional[Union[Path, str]] = None,
        base64: Optional[str] = None,
        data_bytes: Optional[bytes] = None,
        **kwargs,
    ) -> None:
        data = {}
        if sum([bool(url), bool(path), bool(base64)]) > 1:
            raise ValueError("Too many binary initializers!")
        # Web initializer
        data["id"] = id
        data["url"] = url
        # Binary initializer
        if path:
            if isinstance(path, str):
                path = Path(path)
            if not path.exists():
                raise FileNotFoundError(f"{path} is not exist!")
            data["base64"] = b64encode(path.read_bytes())
        elif base64:
            data["base64"] = base64
        elif data_bytes:
            data["base64"] = b64encode(data_bytes)
        super().__init__(**data, **kwargs)

    async def get_bytes(self) -> bytes:
        """尝试获取消息元素的 bytes, 注意, 你无法获取并不包含 url 且不包含 base64 属性的本元素的 bytes.

        Raises:
            ValueError: 你尝试获取并不包含 url 属性的本元素的 bytes.

        Returns:
            bytes: 元素原始数据
        """
        if self.base64:
            return b64decode(self.base64)
        if not self.url:
            raise ValueError("you should offer a url.")
        session = adapter_ctx.get().session
        if not session:
            raise RuntimeError("Unable to get session!")
        async with session.get(self.url) as response:
            response.raise_for_status()
            data = await response.read()
            self.base64 = b64encode(data).decode()
            return data

    def asPersistentString(self, *, binary: bool = True) -> str:
        return (
            f"[mirai:{self.type}:{wrap_bracket(j_dump(self.dict(exclude={'type'})))}]"
            if binary
            else f"[mirai:{self.type}:{wrap_bracket(j_dump(self.dict(exclude={'type', 'base64'})))}]"
        )

    @property
    def uuid(self):
        if self.id:
            return self.id.split(".")[0].strip("/{}").lower()
        return ""

    def __eq__(self, other: "MultimediaElement"):
        if self.type != other.type:
            return False
        if self.uuid and self.uuid == other.uuid:
            return True
        if self.url and self.url == other.url:
            return True
        if self.base64 and self.base64 == other.base64:
            return True
        return False


class Image(MultimediaElement):
    "指示消息中的图片元素"
    type = "Image"
    id: Optional[str] = Field(None, alias="imageId")

    def toFlashImage(self) -> "FlashImage":
        return FlashImage.parse_obj({**self.dict(), "type": "FlashImage"})

    @classmethod
    def fromFlashImage(cls, flash: "FlashImage") -> "Image":
        return cls.parse_obj({**flash.dict(), "type": "Image"})

    def asDisplay(self) -> str:
        return "[图片]"


class FlashImage(Image):
    "指示消息中的闪照元素"
    type = "FlashImage"

    def toImage(self) -> "Image":
        return Image.parse_obj({**self.dict(), "type": "Image"})

    @classmethod
    def fromImage(cls, image: "Image") -> "FlashImage":
        return cls.parse_obj({**image.dict(), "type": "FlashImage"})

    def asDisplay(self) -> str:
        return "[闪照]"


class Voice(MultimediaElement):
    "指示消息中的语音元素"
    type = "Voice"
    id: Optional[str] = Field(None, alias="voiceId")
    length: Optional[int]

    def asDisplay(self) -> str:
        return "[语音]"


def _update_forward_refs():
    """
    Inner function.
    Update the forward references.
    """
    from .chain import MessageChain

    Quote.update_forward_refs(MessageChain=MessageChain)
    ForwardNode.update_forward_refs(MessageChain=MessageChain)



import re
import string
from copy import deepcopy
from typing import (
    Dict,
    Generic,
    Iterable,
    List,
    Optional,
    Tuple,
    Type,
    TypedDict,
    TypeVar,
    Union,
)

from graia.broadcast.entities.dispatcher import BaseDispatcher
from graia.broadcast.exceptions import ExecutionStop
from graia.broadcast.interfaces.dispatcher import DispatcherInterface
from pydantic.utils import Representation

from ...event.message import MessageEvent
from ..chain import MessageChain
from ..element import Element
from .pattern import (
    ArgumentMatch,
    ElementMatch,
    FullMatch,
    Match,
    RegexMatch,
    WildcardMatch,
)
from .util import ArgumentMatchType, TwilightParser, split


class Sparkle(Representation):
    __dict__: Dict[str, Match]

    def __init__(
        self,
        check_args: Iterable[Union[FullMatch, RegexMatch]] = (),
        matches: Optional[Union[Dict[str, Match], Iterable[Tuple[str, Match]]]] = None,
    ):
        if matches is None or isinstance(matches, dict):
            match_map: Dict[str, Match] = matches or {
                k: v for k, v in self.__class__.__dict__.items() if isinstance(v, Match)
            }
        else:
            match_map: Dict[str, Match] = {k: v for k, v in matches}

        if any(
            k.startswith("_") and not re.fullmatch(r"_check_(\d+)", k) or k[0] in string.digits
            for k in match_map.keys()
        ):
            raise ValueError("Invalid Match object name!")

        group_cnt: int = 0
        match_pattern_list: List[str] = []

        self._regex_match_list: List[
            Tuple[str, Union[RegexMatch, FullMatch, WildcardMatch, ElementMatch], int]
        ] = []
        self._args_map: Dict[str, Tuple[ArgumentMatch, str]] = {}
        self._parser = TwilightParser(prog="", add_help=False)
        for k, v in match_map.items():
            if isinstance(v, Match):
                if isinstance(v, ArgumentMatch):
                    self._args_map[v.name] = (v, k)
                    if v.action is ... or self._parser.accept_type(v.action):
                        if "type" not in v.add_arg_data or v.add_arg_data["type"] is MessageChain:
                            v.add_arg_data["type"] = ArgumentMatchType(v, v.regex)
                        if isinstance(v.add_arg_data["type"], type) and issubclass(
                            v.add_arg_data["type"], Element
                        ):
                            raise ValueError("Can't use Element as argument type!")
                    self._parser.add_argument(*v.pattern, **v.add_arg_data)
                else:
                    self._regex_match_list.append((k, v, group_cnt + 1))
                    group_cnt += re.compile(v.gen_regex()).groups
                    match_pattern_list.append(v.gen_regex())
        if (
            not all(v[0].pattern[0].startswith("-") for v in self._args_map.values())
            and self._regex_match_list
        ):
            raise ValueError("ArgumentMatch's pattern can't start with '-'!")

        self._regex_pattern = "".join(match_pattern_list)
        self._regex = re.compile(self._regex_pattern)

        self._check_args = check_args
        self._check_pattern: str = "".join(check_match.gen_regex() for check_match in check_args)
        self._check_regex = re.compile(self._check_pattern)

    def __repr_args__(self):
        return [(k, v) for k, v in self.__dict__.items() if isinstance(v, Match)]

    def run_check(self, string: str) -> List[str]:
        if not self._check_pattern:
            return split(string)
        if match := self._check_regex.match(string):
            return split(string[match.end() :])
        else:
            raise ValueError(f"Not matching regex: {self._check_pattern}")

    def parse_arg_list(self, args: List[str]) -> List[str]:
        if not self._args_map:  # Optimization: skip if no ArgumentMatch
            return args
        namespace, rest = self._parser.parse_known_args(args)
        for arg_name, val_tuple in self._args_map.items():
            match, sparkle_name = val_tuple
            namespace_val = getattr(namespace, arg_name, None)
            if arg_name in namespace.__dict__:
                setattr(
                    self,
                    sparkle_name,
                    match.clone(namespace_val, bool(namespace_val)),
                )
        return rest

    def match_regex(self, elem_mapping: Dict[str, Element], arg_list: List[str]) -> None:
        if self._regex_pattern:
            if regex_match := self._regex.fullmatch(" ".join(arg_list)):
                for name, match, index in self._regex_match_list:
                    current = regex_match.group(index) or ""
                    if isinstance(match, ElementMatch):
                        if current:
                            index = re.fullmatch("\x02(\\d+)_\\w+\x03", current).group(1)
                            result = elem_mapping[int(index)]
                        else:
                            result = None
                    else:
                        result = MessageChain.fromMappingString(current, elem_mapping)
                    if isinstance(match, RegexMatch):
                        setattr(  # sparkle.{name} = toMessageChain(current)
                            self,
                            name,
                            match.clone(
                                result=result,
                                matched=bool(current),
                                re_match=re.match(match.pattern, current),
                            ),
                        )
                    else:
                        setattr(
                            self,
                            name,
                            match.clone(
                                result=result,
                                matched=bool(current),
                            ),
                        )
            else:
                raise ValueError(f"Regex not matching: {self._regex_pattern}")

    def get_help(
        self,
    ) -> str:
        header: List[str] = ["使用方法:"]

        for match in self._check_args:
            header.append(match.get_help())

        for name, match, _ in self._regex_match_list:
            header.append(match.get_help())

        formatter = self._parser._get_formatter()

        formatter.add_usage(None, self._parser._actions, [], prefix=" ".join(header) + " ")

        positional, optional, *_ = self._parser._action_groups
        formatter.start_section("位置匹配")
        for name, match, _ in self._regex_match_list:
            formatter.add_text(f"{name} -> 匹配 {match.get_help()}{' : ' + match.help if match.help else ''}")
        formatter.add_arguments(positional._group_actions)
        formatter.end_section()

        formatter.start_section("参数匹配")
        formatter.add_arguments(optional._group_actions)
        formatter.end_section()

        # determine help from format above
        return formatter.format_help()


T_Sparkle = TypeVar("T_Sparkle", bound=Sparkle)


class _TwilightLocalStorage(TypedDict):
    sparkle: Optional[Sparkle]


class Twilight(BaseDispatcher, Generic[T_Sparkle]):
    """
    暮光.
    """

    def __init__(
        self,
        sparkle: Union[Type[T_Sparkle], T_Sparkle],
        remove_source: bool = True,
        remove_quote: bool = True,
        remove_extra_space: bool = False,
    ):
        """本魔法方法用于初始化本实例.

        Args:
            sparkle (Optional[Type[T_Sparkle]], optional): Sparkle 的子类, 用于生成 Sparkle.
            remove_source (bool, optional): 是否移除消息链中的 Source 元素. 默认为 True.
            remove_quote (bool, optional): 处理时是否要移除消息链的 Quote 元素. 默认为 True.
            remove_extra_space (bool, optional): 是否移除 Quote At AtAll 的多余空格. 默认为 False.
        """
        if isinstance(sparkle, Sparkle):
            self.sparkle_root = sparkle
        else:
            self.sparkle_root = sparkle()
        self.map_params = {
            "remove_source": remove_source,
            "remove_quote": remove_quote,
            "remove_extra_space": remove_extra_space,
        }

    def gen_sparkle(self, chain: MessageChain) -> T_Sparkle:
        sparkle = deepcopy(self.sparkle_root)
        mapping_str, elem_mapping = chain.asMappingString(**self.map_params)
        token = ArgumentMatch.elem_mapping_ctx.set(chain)
        try:
            str_list = sparkle.run_check(mapping_str)
            arg_list = sparkle.parse_arg_list(str_list)
        except Exception:
            raise
        else:
            sparkle.match_regex(elem_mapping, arg_list)
        ArgumentMatch.elem_mapping_ctx.reset(token)
        return sparkle

    def beforeExecution(self, interface: "DispatcherInterface[MessageEvent]"):
        if not isinstance(interface.event, MessageEvent):
            raise ExecutionStop
        local_storage: _TwilightLocalStorage = interface.broadcast.decorator_interface.local_storage
        chain: MessageChain = interface.event.messageChain
        try:
            local_storage["sparkle"] = self.gen_sparkle(chain)
        except:
            raise ExecutionStop

    async def catch(self, interface: "DispatcherInterface[MessageEvent]") -> Optional[T_Sparkle]:
        local_storage: _TwilightLocalStorage = interface.broadcast.decorator_interface.local_storage
        sparkle = local_storage["sparkle"]
        if issubclass(interface.annotation, Sparkle):
            return sparkle
        if issubclass(interface.annotation, Twilight):
            return self
        if issubclass(interface.annotation, Match):
            if hasattr(sparkle, interface.name):
                match: Match = getattr(sparkle, interface.name)
                if isinstance(match, interface.annotation):
                    return match



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['graia',
 'graia.ariadne',
 'graia.ariadne.event',
 'graia.ariadne.message',
 'graia.ariadne.message.parser',
 'graia.ariadne.util']

package_data = \
{'': ['*']}

install_requires = \
['aiohttp>=3.7.4,<4.0.0',
 'graia-broadcast>=0.12.1,!=0.14.0,!=0.14.1',
 'loguru>=0.5.3,<0.6.0',
 'pydantic>=1.8.2,<2.0.0',
 'typing-extensions>=3.10.0,<4.0.0',
 'yarl>=1.6.3,<2.0.0']

setup_kwargs = {
    'name': 'graia-ariadne',
    'version': '0.4.3',
    'description': 'Another elegant framework for mirai and mirai-api-http v2.',
    'long_description': '<div align="center">\n\n# Ariadne\n\n_Another elegant framework for mirai and mirai-api-http v2._\n\n> 接受当下, 面向未来.\n\n</div>\n\n<p align="center">\n  <a href="https://github.com/GraiaProject/Ariadne/blob/master/LICENSE"><img alt="License" src="https://img.shields.io/github/license/GraiaProject/Ariadne"></a>\n  <a href="https://pypi.org/project/graia-ariadne"><img alt="PyPI" src="https://img.shields.io/pypi/v/graia-ariadne" /></a>\n  <a href="https://graia.readthedocs.io/zh-CN/latest"><img alt="docs" src="https://img.shields.io/badge/文档-readthedocs-black" /></a>\n  <a href="https://graiaproject.github.io/Ariadne/"><img alt="API docs" src="https://img.shields.io/badge/API_文档-GitHub_Pages-black"></a>\n  <a href="https://github.com/psf/black"><img src="https://img.shields.io/badge/code%20style-black-000000.svg" alt="black" /></a>\n  <a href="https://pycqa.github.io/isort/"><img src="https://img.shields.io/badge/%20imports-isort-%231674b1?style=flat&labelColor=ef8336" alt="isort"/></a>\n\n</p>\n\n**本项目适用于 mirai-api-http 2.0 以上版本**.\n\n一个适用于 [`mirai-api-http v2`](https://github.com/project-mirai/mirai-api-http) 的 Python 开发框架.\n\n## 安装\n\n`poetry add graia-ariadne`\n\n或\n\n`pip install graia-ariadne`\n\n## 开始使用\n\n```python\nimport asyncio\n\nfrom graia.broadcast import Broadcast\n\nfrom graia.ariadne.app import Ariadne\nfrom graia.ariadne.message.chain import MessageChain\nfrom graia.ariadne.message.element import Plain\nfrom graia.ariadne.model import Friend, MiraiSession\n\nloop = asyncio.new_event_loop()\n\napp = Ariadne(MiraiSession(host="http://localhost:8080", verify_key="ServiceVerifyKey", account=123456789)))\n\n\n@app.broadcast.receiver("FriendMessage")\nasync def friend_message_listener(app: Ariadne, friend: Friend):\n    await app.sendMessage(friend, MessageChain.create([Plain("Hello, World!")]))\n\n\nloop.run_until_complete(app.lifecycle())\n```\n\n更多信息请看 [文档](https://graia.readthedocs.io/zh_CN/latest/).\n\n## 讨论\n\nQQ 交流群: [邀请链接](https://jq.qq.com/?_wv=1027&k=VXp6plBD)\n\n## 文档\n\n[API 文档](https://graiaproject.github.io/Ariadne/)\n[![PDoc Deploy](https://img.shields.io/github/deployments/GraiaProject/Ariadne/github-pages)](https://graiaproject.github.io/Ariadne/)\n\n[文档](https://graia.readthedocs.io/zh_CN/latest/)\n[![Read The Docs Deploy](https://readthedocs.org/projects/graia/badge/?version=latest)](https://graia.readthedocs.io/zh_CN/latest/)\n\n[鸣谢](https://graia.readthedocs.io/zh_CN/latest/appendix/credits)\n\n**如果认为本项目有帮助, 欢迎点一个 `Star`.**\n\n## 协议\n\n本项目以[`GNU AGPLv3`](https://choosealicense.com/licenses/agpl-3.0/) 作为开源协议, 这意味着你需要遵守相应的规则.\n',
    'author': 'BlueGlassBlock',
    'author_email': 'blueglassblock@outlook.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://graia.readthedocs.io/zh_CN/latest',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
